# Generated by roxygen2: do not edit by hand

S3method(coef,FCDCM)
S3method(coef,FCGDINA)
S3method(coef,FCGGUM)
S3method(coef,FCMIRT)
S3method(coef,MGGUM)
S3method(coef,MGPCM)
S3method(coef,MIRT)
S3method(coef,TIRT)
S3method(confint,FCDCM)
S3method(confint,FCGDINA)
S3method(confint,FCGGUM)
S3method(confint,FCMIRT)
S3method(confint,MGGUM)
S3method(confint,MGPCM)
S3method(confint,MIRT)
S3method(confint,TIRT)
S3method(deviance,FCDCM)
S3method(deviance,FCGDINA)
S3method(deviance,FCGGUM)
S3method(deviance,FCMIRT)
S3method(deviance,MGGUM)
S3method(deviance,MGPCM)
S3method(deviance,MIRT)
S3method(deviance,TIRT)
S3method(extract,FCDCM)
S3method(extract,FCGDINA)
S3method(extract,FCGGUM)
S3method(extract,FCMIRT)
S3method(extract,MGGUM)
S3method(extract,MGPCM)
S3method(extract,MIRT)
S3method(extract,TIRT)
S3method(fitted,FCDCM)
S3method(fitted,FCGDINA)
S3method(fitted,FCGGUM)
S3method(fitted,FCMIRT)
S3method(fitted,MGGUM)
S3method(fitted,MGPCM)
S3method(fitted,MIRT)
S3method(fitted,TIRT)
S3method(get.fit.index,FCDCM)
S3method(get.fit.index,FCGDINA)
S3method(get.fit.index,FCGGUM)
S3method(get.fit.index,FCMIRT)
S3method(get.fit.index,MGGUM)
S3method(get.fit.index,MGPCM)
S3method(get.fit.index,MIRT)
S3method(get.fit.index,TIRT)
S3method(get.log_lik,default)
S3method(logLik,FCDCM)
S3method(logLik,FCGDINA)
S3method(logLik,FCGGUM)
S3method(logLik,FCMIRT)
S3method(logLik,MGGUM)
S3method(logLik,MGPCM)
S3method(logLik,MIRT)
S3method(logLik,TIRT)
S3method(nobs,FCDCM)
S3method(nobs,FCGDINA)
S3method(nobs,FCGGUM)
S3method(nobs,FCMIRT)
S3method(nobs,MGGUM)
S3method(nobs,MGPCM)
S3method(nobs,MIRT)
S3method(nobs,TIRT)
S3method(plot,FCDCM)
S3method(plot,FCGDINA)
S3method(plot,FCGGUM)
S3method(plot,FCMIRT)
S3method(plot,MGGUM)
S3method(plot,MGPCM)
S3method(plot,MIRT)
S3method(plot,TIRT)
S3method(predict,FCDCM)
S3method(predict,FCGDINA)
S3method(predict,FCGGUM)
S3method(predict,FCMIRT)
S3method(predict,MGGUM)
S3method(predict,MGPCM)
S3method(predict,MIRT)
S3method(predict,TIRT)
S3method(print,FCDCM)
S3method(print,FCGDINA)
S3method(print,FCGGUM)
S3method(print,FCMIRT)
S3method(print,MGGUM)
S3method(print,MGPCM)
S3method(print,MIRT)
S3method(print,TIRT)
S3method(print,data.FCDCM)
S3method(print,data.FCGDINA)
S3method(print,data.FCGGUM)
S3method(print,data.FCMIRT)
S3method(print,data.MGGUM)
S3method(print,data.MGPCM)
S3method(print,data.MIRT)
S3method(print,data.TIRT)
S3method(print,good.of.fit)
S3method(print,summary.FCDCM)
S3method(print,summary.FCGDINA)
S3method(print,summary.FCGGUM)
S3method(print,summary.FCMIRT)
S3method(print,summary.MGGUM)
S3method(print,summary.MGPCM)
S3method(print,summary.MIRT)
S3method(print,summary.TIRT)
S3method(print,summary.good.of.fit)
S3method(residuals,FCDCM)
S3method(residuals,FCGDINA)
S3method(residuals,FCGGUM)
S3method(residuals,FCMIRT)
S3method(residuals,MGGUM)
S3method(residuals,MGPCM)
S3method(residuals,MIRT)
S3method(residuals,TIRT)
S3method(rotate,MGPCM)
S3method(rotate,MIRT)
S3method(summary,FCDCM)
S3method(summary,FCGDINA)
S3method(summary,FCGGUM)
S3method(summary,FCMIRT)
S3method(summary,MGGUM)
S3method(summary,MGPCM)
S3method(summary,MIRT)
S3method(summary,TIRT)
S3method(summary,good.of.fit)
S3method(update,FCDCM)
S3method(update,FCGDINA)
S3method(update,FCGGUM)
S3method(update,FCMIRT)
S3method(update,MGGUM)
S3method(update,MGPCM)
S3method(update,MIRT)
S3method(update,TIRT)
S3method(vcov,FCDCM)
S3method(vcov,FCGDINA)
S3method(vcov,FCGGUM)
S3method(vcov,FCMIRT)
S3method(vcov,MGGUM)
S3method(vcov,MGPCM)
S3method(vcov,MIRT)
S3method(vcov,TIRT)
export(extract)
export(fit.FCDCM)
export(fit.FCGDINA)
export(fit.FCGGUM)
export(fit.FCMIRT)
export(fit.MGGUM)
export(fit.MGPCM)
export(fit.MIRT)
export(fit.TIRT)
export(get.block.items.from.data)
export(get.block.items.from.data.FCDCM)
export(get.data.from.response)
export(get.data.from.response.FCDCM)
export(get.data.from.response.TIRT)
export(get.fit.index)
export(get.log_lik)
export(get.response.from.data)
export(get.response.from.data.FCDCM)
export(get.response.from.data.TIRT)
export(good.of.fit)
export(model.FCGDINA)
export(rotate)
export(sim.data.FCDCM)
export(sim.data.FCGDINA)
export(sim.data.FCGGUM)
export(sim.data.FCMIRT)
export(sim.data.MGGUM)
export(sim.data.MGPCM)
export(sim.data.MIRT)
export(sim.data.TIRT)
export(summary.good.of.fit)
import(Rcpp)
import(methods)
import(rstantools)
importFrom(RcppParallel,RcppParallelLibs)
importFrom(rstan,sampling)
importFrom(stats,fitted)
importFrom(stats,logLik)
importFrom(stats,nobs)
importFrom(stats,plogis)
importFrom(stats,pnorm)
importFrom(stats,qlogis)
importFrom(stats,qnorm)
importFrom(stats,rlnorm)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,update)
importFrom(utils,combn)
importFrom(utils,flush.console)
importFrom(utils,tail)
useDynLib(ForceChoice, .registration = TRUE)
