% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim.data.TIRT.R
\name{model.TIRT}
\alias{model.TIRT}
\title{Compute TIRT Pairwise Comparison Probabilities}
\usage{
model.TIRT(theta, lambda, psi2, gamma.matrix, Q.matrix, pairs.matrix)
}
\arguments{
\item{theta}{Latent trait matrix.}

\item{lambda}{Factor loading magnitudes per statement. If signed values
are supplied from a fitted object, the function uses \code{abs(lambda)}
and the sign in \code{Q.matrix} carries the direction.}

\item{psi2}{Uniqueness variances per statement.}

\item{gamma.matrix}{Pairwise intercept matrix.}

\item{Q.matrix}{Design Q-matrix.}

\item{pairs.matrix}{Matrix of statement-index pairs.}
}
\value{
Matrix of pairwise choice probabilities:
\eqn{\Phi\{[-\gamma_{ik} + (|\lambda_i|q_i -
  |\lambda_k|q_k)'\theta_j] / \sqrt{\psi_i^2+\psi_k^2}\}}.
}
\description{
Compute TIRT Pairwise Comparison Probabilities
}
\keyword{internal}
