% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim.data.MGGUM.R
\name{model.MGGUM}
\alias{model.MGGUM}
\title{Compute MGGUM Category Response Probabilities}
\usage{
model.MGGUM(theta, par)
}
\arguments{
\item{theta}{\eqn{N \times D} matrix of latent trait values.}

\item{par}{\eqn{I \times (2D + \max K_i)} parameter matrix with columns
\code{a1..aD, delta1..deltaD, tau0, tau1, ...}. Missing category
columns for shorter items should be \code{NA}; \code{tau0} is fixed at 0.}
}
\value{
An \eqn{N \times \sum_i K_i} matrix of stacked category
probabilities. The implementation uses
\eqn{r_{ij} = [\sum_d a_{id}^2(\theta_{jd}-\delta_{id})^2]^{1/2}}
and
\eqn{P(Y_{ij}=k) \propto
  \exp\{k r_{ij}-\psi_{ik}\} +
  \exp\{(2K_i-1-k)r_{ij}-\psi_{ik}\}}.
}
\description{
Compute MGGUM Category Response Probabilities
}
\keyword{internal}
