% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/istem-helpers.R
\name{resolve_model_type}
\alias{resolve_model_type}
\title{Resolve IRT model type with alias support}
\usage{
resolve_model_type(model)
}
\arguments{
\item{model}{Character scalar; the model identifier.}
}
\value{
Lower-case internal model code.
}
\description{
Accepts both internal codes (\code{"m1pl"}, \code{"m2pl"},
\code{"m3pl"}, \code{"m4pl"}) and user-friendly aliases
(\code{"Rasch"}, \code{"1PL"}, \code{"2PL"}, \code{"3PL"},
\code{"4PL"}).
}
\keyword{internal}
