% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools-mcmc.R
\name{extract_theta_stan}
\alias{extract_theta_stan}
\title{Extract theta posterior summary from Stan MCMC output}
\usage{
extract_theta_stan(MCMC.obj, stan.sum, N, D, row_names = NULL)
}
\arguments{
\item{MCMC.obj}{Output of \code{rstan::extract()}.}

\item{stan.sum}{Output of \code{rstan::summary()$summary}.}

\item{N}{Number of persons.}

\item{D}{Number of dimensions.}

\item{row_names}{Row names for the output matrices (typically person IDs).}
}
\value{
A list with components \code{est}, \code{se}, and \code{Rhat}.
}
\description{
Extract theta posterior summary from Stan MCMC output
}
\keyword{internal}
